# Generated by Django 4.0.3 on 2022-03-15 09:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketmanager', '0010_alter_watchconfig_debug_webhooks'),
    ]

    operations = [
        migrations.RenameField(
            model_name='watchconfig',
            old_name='location',
            new_name='structure',
        ),
    ]
