# Generated by Django 4.0.3 on 2022-03-14 06:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketmanager', '0002_alter_structure_created_at_alter_watchconfig_type'),
    ]

    operations = [
        migrations.RenameField(
            model_name='watchconfig',
            old_name='type',
            new_name='eve_type',
        ),
    ]
