# Generated by Django 4.0.3 on 2022-03-19 11:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveuniverse', '0005_type_materials_and_sections'),
        ('marketmanager', '0024_remove_typestatistics_new_eden'),
    ]

    operations = [
        migrations.CreateModel(
            name='StatisticsConfig',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('calculate_regions', models.ManyToManyField(to='eveuniverse.everegion', verbose_name='TypeStatistics Calculation Regions')),
            ],
            options={
                'default_permissions': (),
            },
        ),
    ]
