# Generated by Django 4.0.3 on 2022-03-15 05:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketmanager', '0009_remove_watchconfig_fail_webhooks_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='watchconfig',
            name='debug_webhooks',
            field=models.ManyToManyField(blank=True, help_text='Primarily for Testing/Debugging Purposes. This webhook will receive updates on WatchConfigs that _dont_ notify. Because their Configs didnt Meet/Breach.', related_name='debug_webhooks', to='marketmanager.webhook', verbose_name='Debug Webhook'),
        ),
    ]
