# Generated by Django 4.0.3 on 2022-03-14 12:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveuniverse', '0005_type_materials_and_sections'),
        ('marketmanager', '0004_alter_watchconfig_jita_compare_percent_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='watchconfig',
            name='constellation',
            field=models.ManyToManyField(blank=True, to='eveuniverse.eveconstellation', verbose_name='Constellation'),
        ),
        migrations.AlterField(
            model_name='watchconfig',
            name='location',
            field=models.ManyToManyField(blank=True, to='marketmanager.structure', verbose_name='Structure'),
        ),
        migrations.AlterField(
            model_name='watchconfig',
            name='region',
            field=models.ManyToManyField(blank=True, to='eveuniverse.everegion', verbose_name='Region'),
        ),
        migrations.AlterField(
            model_name='watchconfig',
            name='solar_system',
            field=models.ManyToManyField(blank=True, to='eveuniverse.evesolarsystem', verbose_name='Solar System'),
        ),
        migrations.AlterField(
            model_name='watchconfig',
            name='structure_type',
            field=models.ManyToManyField(blank=True, help_text='Filter by structure Type/Size/Docking (ie, forts/keeps for cap fuel)', related_name='structure_type', to='eveuniverse.evetype', verbose_name='Structure Type Filter'),
        ),
    ]
