# Generated by Django 4.0.3 on 2022-03-18 04:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketmanager', '0017_alter_watchconfig_managed_app_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='typestatistics',
            name='buy_5_percent',
        ),
        migrations.RemoveField(
            model_name='typestatistics',
            name='sell_5_percent',
        ),
        migrations.AddField(
            model_name='typestatistics',
            name='buy_fifth_percentile',
            field=models.DecimalField(decimal_places=2, default=0, max_digits=20, verbose_name='Buy 5th Percentile'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='typestatistics',
            name='sell_fifth_percentile',
            field=models.DecimalField(decimal_places=2, default=0, max_digits=20, verbose_name='Sell 5th Percentile'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='buy_median',
            field=models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Buy Median'),
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='buy_weighted_average',
            field=models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Buy Weighted Average'),
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='sell_median',
            field=models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Sell Median'),
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='sell_weighted_average',
            field=models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Sell Weighted Average'),
        ),
    ]
